@@ -2,10 +2,9 @@
 
 __all__ = ('Chemical',)
 
-import re
 import thermosteam as tmo
 from flexsolve import IQ_interpolation
-from .utils import copy_maybe
+from .utils import copy_maybe, to_searchable_format
 from .properties.identifiers import CAS_from_any, pubchem_db
 from .properties.vapor_pressure import VaporPressure
 from .properties.phase_change import Tb, Tm, Hfus, Hsub, EnthalpyVaporization
@@ -26,7 +25,8 @@ from .properties.free_energy import \
     ExcessEnthalpyRefSolid, ExcessEnthalpyRefLiquid, ExcessEnthalpyRefGas, \
     ExcessEntropyRefSolid, ExcessEntropyRefLiquid, ExcessEntropyRefGas
 from .base import PhaseProperty, PhaseTProperty, PhaseTPProperty, \
-                  display_asfunctor, TDependentModelHandle, TPDependentModelHandle
+                  ThermoModelHandle, TDependentModelHandle, TPDependentModelHandle, \
+                  display_asfunctor
 from .base.units_of_measure import chemical_units_of_measure
 from .properties.dipole import dipole_moment as dipole
 from . import functional as fn 
@@ -42,11 +42,6 @@ from .equilibrium.unifac_data import \
 # from .refractivity import refractive_index
 # from .electrochem import conductivity
 
-# %% Search tools
-
-def to_searchable_format(ID):    
-    return re.sub(r"\B([A-Z])", r" \1", ID).capitalize().replace('_', ' ')
-
 
 # %% Filling missing properties
 
@@ -150,9 +145,9 @@ _chemical_fields = {'\n[Names]  ': _names,
 
 class Chemical:
     """
-    Creates a Chemical object which contains basic information such as 
-    molecular weight and the structure of the species, as well as thermodynamic
-    and transport properties as a function of temperature and pressure.
+    Creates a Chemical object which contains constant chemical properties,
+    as well as thermodynamic and transport properties as a function of 
+    temperature and pressure.
     
     Parameters
     ----------
@@ -291,7 +286,7 @@ class Chemical:
     >>> Water.Psat(373.15)
     101284.55179999319
     >>> # Water.Psat(1000.0) ->
-    >>> # ValueError: no valid model at T=1000.00 K
+    >>> # ValueError: Water has no valid saturated vapor pressure model at T=1000.00 K
     
     Model handles as well as the models themselves have tabulation and plotting methods to help visualize how properties depend on temperature and pressure.
     
@@ -342,10 +337,10 @@ class Chemical:
         
     >>> value = Water.V.l.add_model(1.687e-05)
     ... # Model is appended at the end by default
-    >>> added_model = Water.V.l[-1] 
+    >>> added_model = Water.V.l[-1]
     >>> added_model.show()
     ConstantThermoModel: Constant
-     value: 1.687e-05
+     value (V.l): 1.687e-05 m^3/mol
      Tmin: 0 K
      Tmax: inf K
      Pmin: 0 Pa
@@ -604,6 +599,15 @@ class Chemical:
     def __reduce__(self):
         return unpickle_chemical, (get_chemical_data(self),)
 
+    def __setattr__(self, name, value):
+        isa = isinstance
+        if isa(value, ThermoModelHandle):
+            value.chemical = self
+        elif isa(value, PhaseProperty):
+            for phase in (value.s, value.l, value.g):
+                if isa(phase, ThermoModelHandle): phase.chemical = self
+        super().__setattr__(name, value)
+
     @property
     def ID(self):
         """[str] Identification of chemical."""
@@ -733,7 +737,7 @@ class Chemical:
         self.V = V = Volume(sdata, ldata, gdata)
         
         # Heat capacity
-        Cn = PhaseTProperty(var='Cn')
+        Cn = PhaseTProperty('Cn')
         sdata = (CAS, similarity_variable, MW)
         ldata = (CAS, Tb, Tc, omega, MW, similarity_variable, Cn)
         gdata = (CAS, MW, similarity_variable, iscyclic_aliphatic)
@@ -1020,11 +1024,11 @@ class Chemical:
             getfield = getattr
             single_phase = isinstance(Cn, TDependentModelHandle)
             if single_phase:
-                Cn.add_model(4.18*MW, var='Cn')
+                Cn.add_model(4.18*MW)
                 Cn_phase = Cn
             else:
                 Cn_phase = getfield(Cn, phase_ref)
-                Cn_phase.add_model(4.18*MW, var='Cn')
+                Cn_phase.add_model(4.18*MW)
             self.load_free_energies()
         if not self.H:
             self.load_free_energies()
@@ -1150,14 +1154,14 @@ class Chemical:
         for i in _free_energies: setfield(self, i, None)
         if phase:
             for i in ('kappa', 'mu', 'V'):
-                setfield(self, i, TPDependentModelHandle())
-            self.Cn = TDependentModelHandle()
+                setfield(self, i, TPDependentModelHandle(i))
+            self.Cn = TDependentModelHandle('Cn')
         else:
             for i in ('kappa', 'mu', 'V'):
-                setfield(self, i, PhaseTPProperty())
-            self.Cn = PhaseTProperty()
+                setfield(self, i, PhaseTPProperty(i))
+            self.Cn = PhaseTProperty('Cn')
         for i in ('sigma', 'epsilon', 'Psat', 'Hvap'):
-            setfield(self, i, TDependentModelHandle())
+            setfield(self, i, TDependentModelHandle(i))
         self._locked_state = phase
         self._ID = ID
         self.phase_ref = phase_ref or phase
@@ -1224,7 +1228,7 @@ class Chemical:
                  Tb: 77.355 K
                  Tt: 63.15 K
                  Tc: 126.2 K
-                 Pt: 12527 K
+                 Pt: 12527 Pa
                  Pc: 3.3944e+06 Pa
                  Vc: 8.95e-05 m^3/mol
                  Zc: 0.28953
@@ -1288,7 +1292,7 @@ class Chemical:
         return self.ID
     
     def __repr__(self):
-        return f"<Chemical: {self}>"
+        return f"Chemical('{self}')"
     
 def lock_phase(chemical, phase):
     getfield = getattr
