@@ -30,7 +30,8 @@ def chemical_data_array(chemicals, attr):
 # %% Chemicals
 
 class Chemicals:
-    """Create Chemicals object that contains Chemical objects as attributes.
+    """
+    Create Chemicals object that contains Chemical objects as attributes.
 
     Parameters
     ----------
@@ -67,27 +68,58 @@ class Chemicals:
             else:
                 setattr(self, chem, Chemical(chem))
     
-    def __setattr__(self, ID, chemical):
-        raise TypeError("can't set attribute; use <Chemicals>.append instead")
-    
     def __getnewargs__(self):
         return (tuple(self),)
     
-    def compile(self):
+    def __setattr__(self, ID, chemical):
+        raise TypeError("can't set attribute; use <Chemicals>.append instead")
+    
+    def __setitem__(self, ID, chemical):
+        raise TypeError("can't set item; use <Chemicals>.append instead")
+    
+    def __getitem__(self, key):
         """
-        Cast as a CompiledChemicals object.
+        Return a chemical or a list of chemicals.
         
+        Parameters
+        ----------
+        key : Iterable[str] or str
+              Chemical identifiers.
+              
         Examples
         --------
-        >>> from thermosteam import Chemicals
-        >>> chemicals = Chemicals(['Water', 'Ethanol'])
-        >>> chemicals.compile()
-        >>> chemicals
-        CompiledChemicals([Water, Ethanol])
+        
+        >>> chemicals = Chemicals(['Water', 'Ethanol', 'Propane'])
+        >>> chemicals['Propane', 'Water']
+        [Chemical('Propane'), Chemical('Water')]
         
         """
-        CompiledChemicals._compile(self)
-        setattr(self, '__class__', CompiledChemicals)
+        dct = self.__dict__
+        try:
+            if isinstance(key, str):
+                return dct[key]
+            else:
+                return [dct[i] for i in key]
+        except KeyError as key_error:
+            raise UndefinedChemical(key_error.args[0])
+    
+    def append(self, chemical):
+        """Append a Chemical."""
+        assert isinstance(chemical, Chemical), ("only 'Chemical' objects are allowed, "
+                                               f"not '{type(chemical).__name__}'")
+        ID = chemical.ID
+        assert ID not in self.__dict__, f"{ID} already defined in chemicals"
+        setattr(self, ID, chemical)
+    
+    def extend(self, chemicals):
+        """Extend with more Chemical objects."""
+        if isinstance(chemicals, Chemicals):
+            self.__dict__.update(chemicals.__dict__)
+        else:
+            for chemical in chemicals:
+                assert isinstance(chemical, Chemical), ("only 'Chemical' objects are allowed, "
+                                                        f"not '{type(chemical).__name__}'")
+                setattr(self, chemical.ID, chemical)
     
     def subgroup(self, IDs):
         """
@@ -107,45 +139,22 @@ class Chemicals:
         
         """
         return type(self)([getattr(self, i) for i in IDs])
-        
-    def retrieve(self, IDs):
+    
+    def compile(self):
         """
-        Return a list of chemicals.
+        Cast as a CompiledChemicals object.
         
-        Parameters
-        ----------
-        IDs : Iterable[str]
-              Chemical identifiers.
-              
         Examples
         --------
-        
-        >>> chemicals = Chemicals(['Water', 'Ethanol', 'Propane'])
-        >>> chemicals.retrieve(['Propane', 'Water'])
-        [Chemical('Propane'), Chemical('Water')]
+        >>> from thermosteam import Chemicals
+        >>> chemicals = Chemicals(['Water', 'Ethanol'])
+        >>> chemicals.compile()
+        >>> chemicals
+        CompiledChemicals([Water, Ethanol])
         
         """
-        dct = self.__dict__
-        try:
-            return [dct[i] for i in IDs]
-        except KeyError as key_error:
-            raise UndefinedChemical(key_error.args[0])
-    
-    def append(self, chemical):
-        """Append a Chemical."""
-        assert isinstance(chemical, Chemical), ("only 'Chemical' objects are allowed, "
-                                               f"not '{type(chemical).__name__}'")
-        setattr(self, chemical.ID, chemical)
-    
-    def extend(self, chemicals):
-        """Extend with more Chemical objects."""
-        if isinstance(chemicals, Chemicals):
-            self.__dict__.update(chemicals.__dict__)
-        else:
-            for chemical in chemicals:
-                assert isinstance(chemical, Chemical), ("only 'Chemical' objects are allowed, "
-                                                        f"not '{type(chemical).__name__}'")
-                setattr(self, chemical.ID, chemical)
+        CompiledChemicals._compile(self)
+        setattr(self, '__class__', CompiledChemicals)
     
     kwarray = array = index = indices = must_compile
         
@@ -162,9 +171,10 @@ class Chemicals:
         return f"{type(self).__name__}([{', '.join(self.__dict__)}])"
 
 
-@read_only(methods=('append', 'extend'))
+@read_only(methods=('append', 'extend', '__setitem__'))
 class CompiledChemicals(Chemicals):
-    """Create CompiledChemicals object that contains Chemical objects as attributes.
+    """
+    Create CompiledChemicals object that contains Chemical objects as attributes.
 
     Parameters
     ----------
@@ -204,7 +214,6 @@ class CompiledChemicals(Chemicals):
         
     Examples
     --------
-    
     Create a CompiledChemicals object from chemical identifiers
     
     >>> from thermosteam import CompiledChemicals, Chemical
@@ -242,7 +251,12 @@ class CompiledChemicals(Chemicals):
         return self
     
     def __dir__(self):
-        return self.IDs + tuple(dir(type(self)))
+        return ('append', 'array', 'compile', 'extend', 
+                'get_combustion_reactions', 'get_index',
+                'get_lle_indices', 'get_synonyms',
+                'get_vle_indices', 'iarray', 'ikwarray',
+                'index', 'indices', 'kwarray', 'refresh_constants', 
+                'retrieve', 'set_synonym', 'subgroup') + self.IDs
     
     def __reduce__(self):
         return CompiledChemicals, (self.tuple,)
@@ -360,7 +374,7 @@ class CompiledChemicals(Chemicals):
         CompiledChemicals([Propane, Water])
         
         """
-        chemicals = self.retrieve(IDs)
+        chemicals = self[IDs]
         new = Chemicals(chemicals)
         new.compile()
         for i in new.IDs:
@@ -370,7 +384,8 @@ class CompiledChemicals(Chemicals):
         return new
     
     def get_synonyms(self, ID):
-        """Get all synonyms of a chemical.
+        """
+        Get all synonyms of a chemical.
         
         Parameters
         ----------
@@ -634,7 +649,7 @@ class CompiledChemicals(Chemicals):
         elif IDs is ...:
             return IDs
         else:
-            raise TypeError("only strings, tuples, and ellipsis are valid index keys")
+            raise TypeError("only strings, tuples, and ellipsis are valid index keys")    
     
     def __len__(self):
         return self.size
