@@ -8,10 +8,10 @@ import numpy as np
 from . import indexer
 from . import equilibrium as eq
 from . import functional as fn
-from .base import units_of_measure as thermo_units, convert
+from .base import units_of_measure as thermo_units
 from .exceptions import DimensionError
 from ._thermal_condition import ThermalCondition
-from .utils import Cache, assert_same_chemicals, thermo_user, registered
+from . import utils
 
 __all__ = ('Stream', )
 
@@ -23,8 +23,8 @@ mass_units = indexer.ChemicalMassFlowIndexer.units
 vol_units = indexer.ChemicalVolumetricFlowIndexer.units
 
 # %%
-@thermo_user
-@registered(ticket_name='s')
+@utils.thermo_user
+@utils.registered(ticket_name='s')
 class Stream:
     """
     Create a Stream object that defines material flow rates
@@ -267,8 +267,8 @@ class Stream:
         self._imol = imol
 
     def _init_cache(self):
-        self._bubble_point_cache = Cache(eq.BubblePoint)
-        self._dew_point_cache = Cache(eq.DewPoint)
+        self._bubble_point_cache = utils.Cache(eq.BubblePoint)
+        self._dew_point_cache = utils.Cache(eq.DewPoint)
 
     @classmethod
     def _get_flow_name_and_factor(cls, units):
@@ -412,17 +412,12 @@ class Stream:
 
         """
         value = getattr(self, name)
-        if name == 'T':
-            value = convert(value, 'K', units)
+        units_dct = thermo_units.stream_units_of_measure
+        if name in units_dct:
+            original_units = units_dct[name]
         else:
-            units_dct = thermo_units.stream_units_of_measure
-            if name in units_dct:
-                original_units = units_dct[name]
-            else:
-                raise ValueError(f"no property with name '{name}'")
-            factor = original_units.conversion_factor(units)
-            value *= factor
-        return value
+            raise ValueError(f"no property with name '{name}'")
+        return original_units.convert(value, units)
     
     def set_property(self, name, value, units):
         """
@@ -447,16 +442,12 @@ class Stream:
         202650.0
 
         """
-        if name == 'T':
-            value = convert(value, units, 'K')
+        units_dct = thermo_units.stream_units_of_measure
+        if name in units_dct:
+            original_units = units_dct[name]
         else:
-            units_dct = thermo_units.stream_units_of_measure
-            if name in units_dct:
-                original_units = units_dct[name]
-            else:
-                raise ValueError(f"no property with name '{name}'")
-            factor = original_units.conversion_factor(units)
-            value /= factor
+            raise ValueError(f"no property with name '{name}'")
+        value = original_units.unconvert(value, units)
         setattr(self, name, value)
     
     ### Stream data ###
@@ -705,6 +696,23 @@ class Stream:
     
     ### Stream methods ###
     
+    def in_thermal_equilibrium(self, other):
+        """
+        Return whether or not stream is in thermal equilibrium with
+        another stream.
+        
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> tmo.settings.set_thermo(['Water', 'Ethanol']) 
+        >>> stream = Stream(Water=1, T=300)
+        >>> other = Stream(Water=1, T=300)
+        >>> stream.in_thermal_equilibrium(other)
+        True
+        
+        """
+        return self._TP.in_equilibrium(other._TP)
+    
     def mix_from(self, others):
         """
         Mix all other streams into this one, ignoring its initial contents.
@@ -730,10 +738,7 @@ class Stream:
         elif N_others == 1:
             self.copy_like(others[0])
         else:
-            assert_same_chemicals(self, others)
-            phase = others[0].phase
-            self.phase = phase
-            self.mol[:] = sum([i.mol for i in others])
+            self._imol.mix_from([i._imol for i in others])
             self.H = sum([i.H for i in others])
     
     def split_to(self, s1, s2, split):
@@ -855,8 +860,27 @@ class Stream:
         self._imol.copy_like(other._imol)
         self._TP.copy_like(other._TP)
     
+    def copy_thermal_condition(self, other):
+        """
+        Copy thermal conditions (T and P) of another stream.
+
+        Examples
+        --------
+        >>> import thermosteam as tmo
+        >>> tmo.settings.set_thermo(['Water', 'Ethanol'])
+        >>> s1 = tmo.Stream('s1', Water=2, units='kg/hr')
+        >>> s2 = tmo.Stream('s2', Water=1, units='kg/hr', T=300.00)
+        >>> s1.copy_thermal_condition(s2)
+        >>> s1.show(flow='kg/hr')
+        Stream: s1
+         phase: 'l', T: 300 K, P: 101325 Pa
+         flow (kg/hr): Water  2
+        """
+        self._TP.copy_like(other._TP)
+    
     def copy_flow(self, stream, IDs=..., *, remove=False, exclude=False):
-        """Copy flow rates of stream to self.
+        """
+        Copy flow rates of stream to self.
         
         Parameters
         ----------
@@ -1030,7 +1054,7 @@ class Stream:
         0.0
         
         """
-        self._imol._data[:] = 0
+        self._imol.empty()
     
     ### Equilibrium ###
 
@@ -1080,7 +1104,7 @@ class Stream:
         BubblePoint([Water, Ethanol])
         
         """
-        chemicals = self.chemicals.retrieve(IDs) if IDs else self.vle_chemicals
+        chemicals = self.chemicals[IDs] if IDs else self.vle_chemicals
         bp = self._bubble_point_cache.reload(chemicals, self._thermo)
         return bp
     
@@ -1400,18 +1424,20 @@ class Stream:
     
     @property
     def phases(self):
-        """tuple[str] All phases that may be present."""
-        raise AttributeError(f"'{type(self).__name__}' object has no attribute 'phases'")
+        """tuple[str] All phases present."""
+        return (self.phase,)
     @phases.setter
     def phases(self, phases):
         self.__class__ = ms.MultiStream
         self._imol = self._imol.to_material_indexer(phases)
         self._streams = {}
-        self._vle_cache = Cache(eq.VLE, self._imol, self._TP,
-                                thermo=self._thermo,
-                                bubble_point_cache=self._bubble_point_cache,
-                                dew_point_cache=self._dew_point_cache)
-        self._lle_cache = Cache(eq.LLE, self._imol, self._TP, self._thermo)
+        self._vle_cache = utils.Cache(
+                              eq.VLE, self._imol, self._TP,
+                              thermo=self._thermo,
+                              bubble_point_cache=self._bubble_point_cache,
+                              dew_point_cache=self._dew_point_cache)
+        self._lle_cache = utils.Cache(eq.LLE, self._imol,
+                                      self._TP, self._thermo)
     
     ### Representation ###
     
@@ -1505,11 +1531,9 @@ class Stream:
     _ipython_display_ = show
     
     def print(self):
-        from .utils import repr_IDs_data, repr_kwarg
-        chemical_flows = repr_IDs_data(self.chemicals.IDs, self.mol)
-        price = repr_kwarg('price', self.price)
+        chemical_flows = utils.repr_IDs_data(self.chemicals.IDs, self.mol)
+        price = utils.repr_kwarg('price', self.price)
         print(f"{type(self).__name__}(ID={repr(self.ID)}, phase={repr(self.phase)}, T={self.T:.2f}, "
               f"P={self.P:.6g}{price}{chemical_flows})")
     
-from . import _multi_stream as ms
-del registered
\ No newline at end of file
+from . import _multi_stream as ms
\ No newline at end of file
