@@ -5,6 +5,7 @@ Created on Sat Nov 30 20:18:27 2019
 @author: yoelr
 """
 from . import equilibrium as eq
+from ._chemical import Chemical
 from ._chemicals import Chemicals
 from .mixture import Mixture, new_ideal_mixture
 from .utils import read_only, cucumber
@@ -80,9 +81,33 @@ class Thermo:
         setattr(self, 'Phi', Phi)
         setattr(self, 'PCF', PCF)    
     
+    def as_chemical(self, chemical):
+        isa = isinstance
+        if isa(chemical, str):
+            try: 
+                chemical = self.chemicals.retrieve(chemical)
+            except:
+                chemical = Chemical(chemical)
+        elif not isa(chemical, Chemical):
+            raise ValueError('can only convert string to chemical')
+        return chemical
+    
+    def subgroup(self, IDs):
+        chemicals = self.chemicals.subgroup(IDs)
+        return type(self)(chemicals, None, self.Gamma, self.Phi, self.PCF)
+    
     def __repr__(self):
         return f"{type(self).__name__}(chemicals={self.chemicals}, mixture={self.mixture}, Gamma={self.Gamma.__name__}, Phi={self.Phi.__name__}, PCF={self.PCF.__name__})"
     
-
-    
-    
\ No newline at end of file
+    def show(self):
+        try:
+            mixture_info = self.mixture._info().replace('\n', '\n    ')
+        except:
+            mixture_info = str(self.mixture)
+        print(f"{type(self).__name__}(\n"
+              f"    chemicals={self.chemicals},\n"
+              f"    mixture={mixture_info},\n"
+              f"    Gamma={self.Gamma.__name__},\n"
+              f"    Phi={self.Phi.__name__},\n"
+              f"    PCF={self.PCF.__name__})")
+    _ipython_display_ = show
\ No newline at end of file
