@@ -13,27 +13,29 @@ __all__ = ('PhaseProperty', #'PhasePropertyBuilder',
            'PhaseTPropertyBuilder', 'PhaseTPPropertyBuilder',
            'PhaseZTPProperty', 'PhaseZTProperty')
 
-# %% Utilities
-
-getattr = getattr
+# %% Utilitie
 
 def set_phase_property(phase_property, phase, builder, data):
     if not builder: return
-    model_handle = builder.from_args(data) if hasattr(builder, 'from_args') else builder(data)
-    setattr(phase_property, phase, model_handle)
+    if hasattr(builder, 'from_args'):
+        model_handle = builder.from_args(data)
+        setattr(phase_property, phase, model_handle)
+    else:
+        model_handle = getattr(phase_property, phase)
+        builder.build(model_handle, *data)
     
 
 # %% Abstract class    
 
 @functor_lookalike
 class PhaseProperty:
-    __slots__ = ('s', 'l', 'g', 'var')
+    __slots__ = ('var', 's', 'l', 'g')
     
-    def __init__(self, s=None, l=None, g=None, var=None):
+    def __init__(self, var, s=None, l=None, g=None):
+        self.var = var
         self.s = s
         self.l = l
         self.g = g
-        self.var = var
 
     @property
     def S(self): return self.s
@@ -46,7 +48,8 @@ class PhaseProperty:
         return any((self.s, self.l, self.g)) 
     
     def copy(self):
-        return self.__class__(copy_maybe(self.s),
+        return self.__class__(self.var,
+                              copy_maybe(self.s),
                               copy_maybe(self.l),
                               copy_maybe(self.g))
     __copy__ = copy
@@ -57,11 +60,11 @@ class PhaseProperty:
 class PhaseTProperty(PhaseProperty):
     __slots__ = ()
     
-    def __init__(self, s=None, l=None, g=None, var=None):
-        self.s = TDependentModelHandle() if s is None else s
-        self.l = TDependentModelHandle() if l is None else l
-        self.g = TDependentModelHandle() if g is None else g
+    def __init__(self, var, s=None, l=None, g=None):
         self.var = var
+        self.s = TDependentModelHandle(var + '.s') if s is None else s
+        self.l = TDependentModelHandle(var + '.l') if l is None else l
+        self.g = TDependentModelHandle(var + '.g') if g is None else g
     
     def __call__(self, phase, T):
         return getattr(self, phase)(T)
@@ -70,11 +73,11 @@ class PhaseTProperty(PhaseProperty):
 class PhaseTPProperty(PhaseProperty):
     __slots__ = ()
     
-    def __init__(self, s=None, l=None, g=None, var=None):
-        self.s = TPDependentModelHandle() if s is None else s
-        self.l = TPDependentModelHandle() if l is None else l
-        self.g = TPDependentModelHandle() if g is None else g
+    def __init__(self, var, s=None, l=None, g=None):
         self.var = var
+        self.s = TPDependentModelHandle(var + '.s') if s is None else s
+        self.l = TPDependentModelHandle(var + '.l') if l is None else l
+        self.g = TPDependentModelHandle(var + '.g') if g is None else g
     
     def __call__(self, phase, T, P):
         return getattr(self, phase)(T, P)
@@ -105,16 +108,16 @@ class PhaseZTPProperty(PhaseProperty):
 # %% Builders
 
 class PhasePropertyBuilder:
-    __slots__ = ('s', 'l', 'g', 'var')
+    __slots__ = ('var', 's', 'l', 'g')
     
-    def __init__(self, s, l, g, var):
+    def __init__(self, var, s, l, g):
+        self.var = var
         self.s = s
         self.l = l
         self.g = g
-        self.var = var
         
     def __call__(self, sdata, ldata, gdata, phase_property=None):
-        if phase_property is None: phase_property = self.PhaseProperty(var=self.var) 
+        if phase_property is None: phase_property = self.PhaseProperty(self.var) 
         phases = ('s', 'g', 'l')
         builders = (self.s, self.g, self.l)
         phases_data = (sdata, gdata, ldata)
