@@ -4,7 +4,7 @@ Created on Wed Mar 20 18:40:05 2019
 
 @author: yoelr
 """
-from ..utils.decorator_utils import thermo_user
+from ..utils import thermo_user, njitable
 from scipy.optimize import differential_evolution
 from .._thermal_condition import ThermalCondition
 from .vle import VLE
@@ -12,18 +12,39 @@ import numpy as np
 
 __all__ = ('LLE',)
 
-def gibbs_free_energy_of_liquid(l, T, f_gamma):
-    l_sum = l.sum()
-    if l_sum:
-        x = l / l_sum
+def liquid_activities(mol_l, T, f_gamma):
+    total_mol_l = mol_l.sum()
+    if total_mol_l:
+        x = mol_l / total_mol_l
         gamma = f_gamma(x, T)
         xgamma = x * gamma
-        xgamma[xgamma <= 0] = 1
-        g_mix = (l * np.log(xgamma)).sum()
     else:
-        g_mix = 0
+        xgamma = np.ones_like(mol_l)
+    return xgamma
+
+@njitable
+def gibbs_free_energy_of_liquid(mol_l, xgamma):
+    xgamma[xgamma <= 0] = 1
+    g_mix = (mol_l * np.log(xgamma)).sum()
     return g_mix
 
+def lle_objective_function(mol_l, mol, T, f_gamma):
+    mol_L = mol - mol_l
+    xgamma_l = liquid_activities(mol_l, T, f_gamma)
+    xgamma_L = liquid_activities(mol_L, T, f_gamma)
+    g_mix_l = gibbs_free_energy_of_liquid(mol_l, xgamma_l)
+    g_mix_L = gibbs_free_energy_of_liquid(mol_L, xgamma_L)
+    g_mix = g_mix_l + g_mix_L
+    return g_mix
+
+def solve_lle_liquid_mol(mol, T, f_gamma, **differential_evolution_options):
+    args = (mol, T, f_gamma)
+    bounds = np.zeros([mol.size, 2])
+    bounds[:, 1] = mol
+    result = differential_evolution(lle_objective_function, bounds, args,
+                                    **differential_evolution_options)
+    return result.x
+
 @thermo_user
 class LLE:
     """
@@ -58,19 +79,15 @@ class LLE:
     
     """
     __slots__ = ('_thermo', # [float] Thermo object for estimating mixture properties.
-                 '_gamma', # [ActivityCoefficients] Estimates activity coefficients of a liquid.
                  '_imol', # [MaterialIndexer] Stores vapor and liquid molar data.
-                 '_TP', # [ThermalCondition] T values are stored here.
-                 '_index', # [1d array] Index of chemicals in equilibrium.
-                 '_mol') # [Chemical] Single chemical in equilibrium.
-    
+                 '_TP', # [ThermalCondition] T and P values are stored here.
+)
     differential_evolution_options = {'seed': 0}
     
     def __init__(self, imol, thermal_condition=None, thermo=None):
         self._load_thermo(thermo)
         self._TP = thermal_condition or ThermalCondition(298.15, 101325.)
         self._imol = imol
-        self._index = ()
     
     def __call__(self, T, P=None):
         """
@@ -84,45 +101,28 @@ class LLE:
             Operating pressure [Pa].
             
         """
-        self._setup()
         TP = self._TP
         TP.T = T
         if P: TP.P = P
-        if self._mol.sum():
-            l = self._solve_l(T)
-            index = self._index
-            imol = self._imol
-            imol['l'][index] = l
-            imol['L'][index] = self._mol - l
+        imol = self._imol
+        mol, index, lle_chemicals = self.get_liquid_mol_data()
+        total_mol = mol.sum()
+        if total_mol:
+            gamma = self.thermo.Gamma(lle_chemicals)
+            mol_l = solve_lle_liquid_mol(mol, T, gamma,
+                                         **self.differential_evolution_options)
+            imol['l'][index] = mol_l
+            imol['L'][index] = mol - mol_l
     
-    def _setup(self):
+    def get_liquid_mol_data(self):
         # Get flow rates
         imol = self._imol
         mol = imol['l'] + imol['L']
-        
-        # Set up indices for both equilibrium and non-equilibrium species
-        chemicals = self.chemicals
-        index = chemicals.get_lle_indices(mol > 0)
-        chemicals = chemicals.tuple
+        index = self.chemicals.get_lle_indices(mol > 0)
+        mol = mol[index]
+        chemicals = self.chemicals.tuple
         lle_chemicals = [chemicals[i] for i in index]
-        self._gamma = self.thermo.Gamma(lle_chemicals)
-        self._index = index
-        self._mol = mol[index]
-     
-    def _gibbs_free_energy(self, l, T):
-        f_gamma = self._gamma
-        L = self._mol - l
-        return (gibbs_free_energy_of_liquid(L, T, f_gamma)
-                + gibbs_free_energy_of_liquid(l, T, f_gamma))
-    
-    def _solve_l(self, T):
-        mol = self._mol
-        args = (T,)
-        bounds = np.zeros([mol.size, 2])
-        bounds[:, 1] = mol
-        result = differential_evolution(self._gibbs_free_energy, bounds, args,
-                                        **self.differential_evolution_options)
-        return result.x
+        return mol, index, lle_chemicals
     
     imol = VLE.imol
     thermal_condition = VLE.thermal_condition
