@@ -878,12 +878,15 @@ class GroupCounts(dict):
     __slots__ = ()
     
     def __init_subclass__(cls):
-        cls.subgroups_by_name = {i.group:i for i in cls.subgroups_by_id.values()}
+        subgroups_by_id = cls.subgroups_by_id
+        cls.ids_by_group_name = {subgroup.group: key
+                                 for key, subgroup in subgroups_by_id.items()}
     
     def set_group_counts_by_name(self, group_counts):
+        self.clear()
+        ids_by_group_name = self.ids_by_group_name
         for name, count in group_counts.items():
-            subgroup = self.subgroups_by_name[name]
-            key = subgroup.main_group_id
+            key = ids_by_group_name[name]
             if key in self:
                 self[key] += count
             else:
