@@ -338,6 +338,7 @@ class Mixture:
     def xsolve_T(self, phase_data, H, T_guess, P):
         """Solve for temperature in Kelvin"""
         # First approximation
+        phase_data = tuple(phase_data)
         H_guess = self.xH(phase_data, T_guess, P)
         if (H - H_guess) < 1e-3: return T_guess
         Cn = self.xCn(phase_data, T_guess)
@@ -425,6 +426,16 @@ class Mixture:
     def __repr__(self):
         return f"{type(self).__name__}({repr(self.description)}, ..., rigorous_energy_balance={self.rigorous_energy_balance}, include_excess_energies={self.include_excess_energies})"
     
+    def _info(self):
+        return (f"{type(self).__name__}(\n"
+                f"    description={repr(self.description)}, ...\n"
+                f"    rigorous_energy_balance={self.rigorous_energy_balance},\n"
+                f"    include_excess_energies={self.include_excess_energies})")
+    
+    def show(self):
+        print(self._info())
+    _ipython_display_ = show
+        
     
 def new_ideal_mixture(chemicals,
                       rigorous_energy_balance=True,
