@@ -8,13 +8,16 @@ from . import chemicals_user
 from . import thermo_user
 from . import read_only
 from . import registered
+from . import speed_up
 
 __all__ = (*chemicals_user.__all__,
            *thermo_user.__all__,
            *read_only.__all__,
-           *registered.__all__)
+           *registered.__all__,
+           *speed_up.__all__)
 
 from .registered import *
 from .chemicals_user import *
 from .thermo_user import *
-from .read_only import *
\ No newline at end of file
+from .read_only import *
+from .speed_up import *
\ No newline at end of file
